
# line 25 "parser.y"


#include <math.h>
#include "sdbInt.h"

/*
 * Definitions
 */
#define YYERROR_VERBOSE 1

/*
 * Linked lists to keep track of nodes allocated by
 * the parser.  Used for error recovery.
 */
typedef struct SdbNodeList {
  SdbNode            *node;
  struct SdbNodeList *next;
} SdbNodeList;

typedef struct SdbCatList {
  SdbCat             *cat;
  struct SdbCatList  *next;
} SdbCatList;

static SdbNodeList   *safeNodeList = NULL;
static SdbCatList    *safeCatList = NULL;

/*
 * Local variables used to pass the expression string to the
 * lexical analyzer.
 */
static int            sdbTextPos, sdbTextLen;
static char          *sdbTextBuff=NULL;

/*
 * Local variables that pass information into and out of the
 * parser.  Unfortunately, these are required because yyparse
 * does not allow arguments to be passed in.  This results in
 * a non-reentrant program!
 */
static Tcl_HashTable *htbl; 
static Tcl_Interp    *CurrentInterp=NULL;
static SdbNode       *expr;

/* Local prototypes */
static void           Sdberror _ANSI_ARGS_((char*));
int            Sdblex _ANSI_ARGS_((void));
static SdbNode       *safeGetNode _ANSI_ARGS_((void));
static SdbCat        *safeGetCat _ANSI_ARGS_((void));
static void           safeFreeMemory _ANSI_ARGS_((void));
static void           safeFreeLists  _ANSI_ARGS_((void));
int                   Sdbparse _ANSI_ARGS_((void));

#define YY_DECL static int     Sdblex _ANSI_ARGS_((void))


/*
 *------------------------------------------------------------
 *
 * Sdbwrap --
 *
 *      Called by the lexical analyzer at the end of input.
 *
 * Results:
 *      Tells the analyzer that there is no more input.
 *
 * Side effects:
 *      None.
 *
 *------------------------------------------------------------
 */
static int 
Sdbwrap _ANSI_ARGS_((void))
{
   return 1;
}


/*
 * The following memeory allocation routines are a pain in the
 * butt.  Unfortunately, they are required.  The bison/yacc parser
 * blows away the stack when it encounters an error, and that
 * is where the addresses to intermediate structures are stored.
 * The result is memory leaks when an error occurs.  My solution
 * is to keep track of the memory that is allocated.  If the
 * parse is successful, the list of allocations is erased.  If the parse
 * fails then the lists are traversed and the structures are freed.
 */

/*
 *------------------------------------------------------------
 *
 * safeGetNode --
 *
 *      Called by the parser to get a node and remember the
 *      location
 *
 * Results:
 *      Returns an empty node.
 *
 * Side effects:
 *      Adds the address of the node onto the node list.
 *
 *------------------------------------------------------------
 */
static SdbNode        *
safeGetNode _ANSI_ARGS_((void))
{
  SdbNode *node=SdbGetNode();
  SdbNodeList *memo=(SdbNodeList*)ckalloc(sizeof(SdbNodeList));
  memo->node=node;
  memo->next = safeNodeList;
  safeNodeList = memo;
  return(node);
}

static SdbCat         *
safeGetCat _ANSI_ARGS_((void))
{
  SdbCat *cat=(SdbCat*) ckalloc(sizeof(SdbCat));
  SdbCatList *memo=(SdbCatList*)ckalloc(sizeof(SdbCatList));
  memo->cat=cat;
  memo->next = safeCatList;
  safeCatList = memo;
  return(cat);
}


/*
 *------------------------------------------------------------
 *
 * safeFreeMemory --
 *
 *      Called by the parser free all nodes that have been
 *      allocated in this parse.
 *
 * Results:
 *      None
 *
 * Side effects:
 *      None
 *
 *------------------------------------------------------------
 */
static void
safeFreeMemory _ANSI_ARGS_((void))
{
  SdbNodeList *memo=safeNodeList;
  SdbCatList *catList=safeCatList;
  SdbNode *tree;
  
  while(memo) {
    tree=memo->node;
    switch (tree->type) {
    case hsgCodeList:
        ckfree(tree->content.list.pntr);
        break;

    case curveFnct:{
            if (tree->content.yieldHandle->list)
	        ckfree(tree->content.yieldHandle->list);
            ckfree(tree->content.yieldHandle);
            break;
        }

    default:
        break;
    }
    ckfree(tree);
    memo = memo->next;
  }
  while(catList) {
    ckfree(catList->cat);
    catList = catList->next;
  }
}

static void
safeFreeLists  _ANSI_ARGS_((void))
{  
  SdbNodeList *memo=safeNodeList, *temp;
  SdbCatList *catList=safeCatList, *temp2;
  while(memo) {
    temp=memo->next;
    ckfree(memo);
    memo = temp;
  }
  safeNodeList = NULL;
  while(catList) {
    temp2=catList->next;
    ckfree(catList);
    catList = temp2;
  }
  safeCatList = NULL;
}
    


# line 245 "parser.y"
typedef union
#ifdef __cplusplus
	YYSTYPE
#endif
 {
    SdbNode      *node;
    int          integer;
    SdbOpIndex   op;
    double       value;
    double       (*fnctptr)();
    char         *string;
    SdbCat       *classList;
} YYSTYPE;
# define IF 257
# define IN 258
# define RESERVED 259
# define FORMAT 260
# define INTERPOLATE 261
# define EOS 262
# define EOI 263
# define CONNECTOR 264
# define RANDOM 265
# define FLOAT 266
# define C_FNCT 267
# define S_FNCT 268
# define D_FNCT 269
# define HSG_FNCT 270
# define HSG_CODE_SYM 271
# define HSG_NUM_SYM 272
# define UNKNOWN 273
# define MULTI_STRING 274
# define STRING 275
# define INT 276
# define OR 277
# define AND 278
# define numc 279
# define EQ 280
# define NE 281
# define LT 282
# define LE 283
# define GT 284
# define GE 285
# define MINUS 286
# define PLUS 287
# define MUL 288
# define DIV 289
# define MOD 290
# define SIG 291
# define POW 292
# define NOT 293
# define UNARY 294

#include <malloc.h>
#include <memory.h>
#include <values.h>

#ifdef __cplusplus

#ifndef yyerror
	void yyerror(const char *);
#endif

#ifndef yylex
#ifdef __EXTERN_C__
	extern "C" { int yylex(void); }
#else
	int yylex(void);
#endif
#endif
	int yyparse(void);

#endif
#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
extern int yychar;
extern int yyerrflag;
YYSTYPE yylval;
YYSTYPE yyval;
typedef int yytabelem;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif
#if YYMAXDEPTH > 0
int yy_yys[YYMAXDEPTH], *yys = yy_yys;
YYSTYPE yy_yyv[YYMAXDEPTH], *yyv = yy_yyv;
#else	/* user does initial allocation */
int *yys;
YYSTYPE *yyv;
#endif
static int yymaxdepth = YYMAXDEPTH;
# define YYERRCODE 256

# line 1232 "parser.y"
 /* --------------------------------------------------------- */

static SdbVariable *VarPtr=NULL;

#include "lex.Sdb.c"


SdbNode*
SdbParse(interp, dbPtr, string) 
  Tcl_Interp *interp; 
  SdbFile *dbPtr;
  char *string;
{
  int result;

  /* set the following static variables so that they may be accessed
     from within the parser. */

  VarPtr = dbPtr->varPtr;
  CurrentInterp = interp;
  htbl=dbPtr->htbl;
  expr = NULL;

  analyze_buff_init(string);
  result=Sdbparse();

  /*
   * Cheack the result returned from SdbParse.  If it is non-zero
   * there has been an error.  In this case, erase all allocated
   * memeory and return NULL.
   */
  if (result) {
    safeFreeMemory();
    safeFreeLists();
    return NULL;
  }

  safeFreeLists();
  return expr;

}
yytabelem yyexca[] ={
-1, 1,
	0, -1,
	-2, 0,
-1, 123,
	63, 0,
	-2, 14,
-1, 125,
	63, 0,
	-2, 20,
	};
# define YYNPROD 83
# define YYLAST 1490
yytabelem yyact[]={

    10,    25,    35,    40,    36,    39,    37,    38,    45,    44,
    46,    47,    49,    48,    48,    45,    44,    46,    47,    49,
   114,    48,    46,    47,    49,   113,    48,    35,    40,    36,
    39,    37,    38,   154,    10,    25,   211,   197,   210,   209,
   208,   205,   204,   203,   199,   106,   107,   104,   105,    75,
    31,    29,    74,    32,    59,     3,    34,    30,    28,   117,
   116,    53,   131,    41,    10,    25,   159,   186,   138,    58,
     4,   148,   158,   157,   149,   155,   152,   129,   147,   143,
   168,   140,    97,    80,    67,    66,    34,    65,    64,    63,
    62,    61,    60,   133,     1,    10,    25,    79,   182,    78,
    96,    93,    94,    91,    95,    92,    43,    14,    98,   101,
    54,   162,   132,   153,     0,     0,     0,    34,     0,     0,
     0,     0,     0,     0,   100,    10,    25,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,   126,    25,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,   139,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,   124,    25,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,   122,    25,     0,    21,     0,
    12,    19,     0,    57,     0,    20,    26,    17,    16,    18,
    15,     7,     9,    13,   212,    11,    27,    34,     0,     0,
    35,    40,    36,    39,    37,    38,    22,    23,     0,     0,
     5,     0,    21,    24,    12,    19,    50,     2,     0,    20,
    26,    17,    16,    18,    15,     7,     9,    13,     0,    11,
    27,   119,    25,    35,    40,    36,    39,    37,    38,     0,
    22,    23,    21,   130,    12,    19,     0,    24,     0,    20,
    26,    17,    16,    18,    15,     7,    69,    13,   128,    11,
    27,    10,    25,    35,    40,    36,    39,    37,    38,     0,
    22,    23,     0,    21,     0,    12,    19,    24,    57,     0,
    20,    26,    17,    16,    18,    15,     7,     9,    13,     0,
    11,    27,    10,    25,    35,    40,    36,    39,    37,    38,
     0,    22,    23,    21,     0,    12,    19,   206,    24,     0,
    20,    26,    17,    16,    18,    15,    71,    69,    13,     0,
    11,    27,     0,     0,     0,     0,     0,   200,     0,    50,
     0,    22,    23,    21,     0,    12,    19,     0,    24,     0,
    20,    26,    17,    16,    18,    15,    71,    69,    13,    50,
    11,    27,     0,     0,     0,     0,     0,   198,     0,     0,
     0,    22,    23,    21,     0,    12,    19,     0,    24,     0,
    20,    26,    17,    16,    18,    15,    71,    69,    13,    50,
    11,    27,     0,     0,     0,     0,     0,   194,     0,     0,
     0,    22,    23,    21,     0,    12,    19,     0,    24,     0,
    20,    26,    17,    16,    18,    15,    71,    69,    13,    50,
    11,    27,     0,     0,    35,    40,    36,    39,    37,    38,
     0,    22,    23,     0,   189,    50,     0,     0,    24,     0,
    52,    51,     0,    35,    40,    36,    39,    37,    38,    45,
    44,    46,    47,    49,     0,    48,    50,     0,     0,    21,
   181,    12,    19,     0,     0,     0,    20,    26,    17,    16,
    18,    15,    71,    69,    13,     0,    11,    27,     0,    50,
     0,     0,     0,     0,     0,     0,   180,    22,    23,    21,
     0,    12,    19,     0,    24,     0,    20,    26,    17,    16,
    18,    15,     7,     9,    13,    50,    11,    27,     0,     0,
     0,     0,     0,     0,     0,     0,     0,    22,    23,    50,
    21,   174,    12,    19,    24,     0,     0,    20,    26,    17,
    16,    18,    15,     7,    69,    13,     0,    11,    27,     0,
    50,     0,     0,     0,     0,   165,     0,     0,    22,    23,
     0,     0,     0,    52,    51,    24,    35,    40,    36,    39,
    37,    38,    45,    44,    46,    47,    49,    50,    48,     0,
   137,     0,     0,    52,    51,     0,    35,    40,    36,    39,
    37,    38,    45,    44,    46,    47,    49,     0,    48,    50,
     0,     0,     0,     0,     0,   135,     0,     0,     0,     0,
     0,     0,     0,    52,    51,     0,    35,    40,    36,    39,
    37,    38,    45,    44,    46,    47,    49,    50,    48,     0,
     0,     0,     0,     0,     0,   102,     0,     0,     0,     0,
     0,     0,     0,    52,    51,     0,    35,    40,    36,    39,
    37,    38,    45,    44,    46,    47,    49,    50,    48,    52,
    51,     0,    35,    40,    36,    39,    37,    38,    45,    44,
    46,    47,    49,     0,    48,   202,     0,     0,     0,     0,
    52,    51,     0,    35,    40,    36,    39,    37,    38,    45,
    44,    46,    47,    49,     0,    48,     0,    34,   201,     0,
     0,     0,     0,    52,    51,   195,    35,    40,    36,    39,
    37,    38,    45,    44,    46,    47,    49,     0,    48,     0,
    34,   193,     0,     0,     0,     0,     0,    34,     0,    52,
    51,     0,    35,    40,    36,    39,    37,    38,    45,    44,
    46,    47,    49,    34,    48,     0,    35,    40,    36,    39,
    37,    38,    45,    44,    46,    47,    49,     0,    48,   178,
     0,     0,     0,     0,    52,    51,   177,    35,    40,    36,
    39,    37,    38,    45,    44,    46,    47,    49,    34,    48,
     0,     0,     0,     0,     0,    34,   175,     0,     0,     0,
     0,    52,    51,   170,    35,    40,    36,    39,    37,    38,
    45,    44,    46,    47,    49,    34,    48,     0,   166,     0,
     0,     0,    34,    52,    51,   141,    35,    40,    36,    39,
    37,    38,    45,    44,    46,    47,    49,    34,    48,     0,
     0,     0,     0,   136,    34,     0,     0,     0,     0,   134,
     0,    52,    51,     0,    35,    40,    36,    39,    37,    38,
    45,    44,    46,    47,    49,    34,    48,     0,     0,     0,
     0,    34,     0,   103,     0,     0,     0,     0,     0,     0,
     0,    52,    51,     0,    35,    40,    36,    39,    37,    38,
    45,    44,    46,    47,    49,    34,    48,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,    35,    40,    36,    39,    37,    38,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,    35,    40,    36,
    39,    37,    38,     0,    35,    40,    36,    39,    37,    38,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
    35,    40,    36,    39,    37,    38,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,    35,    40,    36,    39,    37,
    38,     0,    35,    40,    36,    39,    37,    38,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,    35,    40,    36,    39,    37,    38,     0,    35,
    40,    36,    39,    37,    38,     0,     0,     0,     0,     0,
     0,     0,     0,     0,    35,    40,    36,    39,    37,    38,
     0,    35,    40,    36,    39,    37,    38,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,    35,    40,    36,    39,    37,    38,    35,    40,
    36,    39,    37,    38,     0,    77,     8,     0,     0,     0,
     0,     0,     0,     0,     0,     0,    55,     0,     0,     0,
     0,     0,    35,    40,    36,    39,    37,    38,    68,    72,
    73,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,    81,    83,    84,
    85,    86,    87,    88,    89,    90,     0,     0,     0,    99,
     8,     0,     0,     0,     0,     0,     0,     0,   110,     0,
   112,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,   118,     0,     0,     0,    42,
     0,     0,     0,     0,     0,     0,    33,     0,     0,     0,
   121,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,    99,     0,     0,    55,     0,     0,    55,     0,
    55,     0,    55,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,   150,     0,     0,    33,     0,     0,     0,
     0,     0,     0,   160,     0,   163,     0,     0,     0,     0,
    33,     0,     0,   171,   172,     0,    33,     0,     0,     0,
   120,     0,    33,     0,     0,     0,   179,     0,     0,     0,
   184,     0,     0,     0,   188,     0,   190,   127,     0,     0,
     0,     0,   196,     0,     0,     0,     0,     0,    33,    33,
     0,    33,     0,   207,     0,    33,    70,     6,     0,   142,
     0,     0,   144,    33,   145,    33,   146,    56,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
    76,    33,     0,     0,     0,     0,    33,     0,    82,     0,
     0,    33,     0,     0,    33,     0,     0,    33,     0,    33,
     6,     6,     0,    33,     0,     0,    33,   108,   109,     0,
   111,     0,     0,    33,   115,    33,     0,    33,     0,     0,
     0,    33,    33,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,   123,   125,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     6,     0,     0,     0,    82,     0,    56,     0,     0,    56,
     0,    56,     0,    56,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,   151,     0,     0,     0,     0,
   156,     0,     0,     0,   161,     0,   164,     0,     0,   167,
     0,     0,   169,     0,     0,     0,   173,     0,     0,     0,
     0,     0,     0,   176,     0,     0,     0,     0,     0,     0,
     0,   183,   185,     0,   187,     0,     0,     0,   191,   192 };
yytabelem yypact[]={

    -6,-10000000,-10000000,  -205,  -206,  -210,   174,     2,   402,     0,
    55,-10000000,    52,-10000000,-10000000,    51,    50,    49,    48,    47,
    45,    44,    85,    85,    85,-10000000,-10000000,-10000000,-10000000,  -211,
-10000000,  -214,-10000000,    85,    43,-10000000,-10000000,-10000000,-10000000,-10000000,
-10000000,    85,    85,    85,    85,    85,    85,    85,    85,    85,
    42,-10000000,-10000000,   292,   261,   614,   842,-10000000,  -215,  -217,
    85,    85,    85,    85,    85,  -250,  -255,    85,-10000000,    -1,
   174,    -2,-10000000,-10000000,-10000000,-10000000,-10000000,   402,-10000000,   231,
  -253,   402,   174,  -271,   486,  -266,  -266,  -279,  -279,  -279,
  -279,-10000000,-10000000,-10000000,   175,   145,   115,  -253,-10000000,   402,
    36,    21,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,   174,   818,
   584,   812,   556,    24,    40,   791,    85,    85,  -278,   -40,
    35,  -278,   -40,  -253,   -40,  -253,   -40,    34,-10000000,-10000000,
-10000000,-10000000,    30,-10000000,-10000000,-10000000,-10000000,    85,    85,    32,
-10000000,  -241,    31,    85,    29,    28,    22,    85,-10000000,    85,
   534,   784,    85,    39,-10000000,    85,   769,    85,    85,    85,
   507,   762,-10000000,   402,   174,-10000000,    85,   742,-10000000,   735,
    85,   472,   446,    54,    85,    85,    23,    85,    85,   423,
    85,    85,    85,   700,   386,   684,    85,    -7,   356,  -220,
   326,   677,   654,  -221,  -222,  -223,   306,    85,  -224,-10000000,
  -225,  -226,  -228,-10000000,-10000000,-10000000,-10000000,   193,-10000000,-10000000,
-10000000,-10000000,-10000000 };
yytabelem yypgo[]={

     0,  1105,    69,   113,    54,  1306,   112,   111,   110,   107,
  1189,   106,   105,   104,   103,   102,   101,   100,    99,    97,
    94,    93 };
yytabelem yyr1[]={

     0,    20,    20,    20,    20,    20,    20,    21,     9,     9,
     2,     2,     4,     4,    12,    13,    13,    14,    15,    15,
    16,    17,    17,    18,    19,    19,     8,     8,     8,     8,
     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     3,     5,
     5,     5,     5,     5,     5,     5,     5,     6,     6,     7,
     7,    10,    10,    10,    10,    10,    10,    11,    11,     1,
     1,     1,     1 };
yytabelem yyr2[]={

     0,     3,     5,     5,     5,     7,     7,     1,     3,     3,
     3,     7,     3,     7,     5,    17,    19,     5,    17,    19,
     5,    17,    19,     5,    17,    19,     5,     5,     7,     7,
     7,     3,     9,     3,     3,     9,     9,     9,    13,    21,
    23,     9,    13,     7,     7,     7,     7,     7,     7,     7,
     7,     7,     5,     5,     5,     7,     7,     7,     3,     7,
     9,     3,     3,     7,     7,    11,     7,     7,     3,     3,
     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
     5,     7,     7 };
yytabelem yychk[]={

-10000000,   -20,   263,    -4,    -2,   256,    -5,   271,    -1,   272,
    40,   275,   260,   273,    -9,   270,   268,   267,   269,   261,
   265,   258,   286,   287,   293,    41,   266,   276,   263,   256,
   263,   256,   263,   -10,    63,   280,   282,   284,   285,   283,
   281,    61,   -10,   -11,   287,   286,   288,   289,   292,   290,
    63,   278,   277,    61,    -8,    -1,    -5,   263,    -2,    -4,
    40,    40,    40,    40,    40,    40,    40,    40,    -1,   272,
    -5,   271,    -1,    -1,   263,   263,    -5,    -1,   -18,   -19,
    40,    -1,    -5,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
    -1,   -14,   -12,   -16,   -15,   -13,   -17,    40,    -4,    -1,
    -2,    -4,    41,    41,   262,   263,   262,   263,    -5,    -5,
    -1,    -5,    -1,   275,   275,    -5,    61,    61,    -1,    40,
   -10,    -1,    40,    -5,    40,    -5,    40,   -10,   262,    41,
   262,    41,    -6,   -21,    41,    41,    41,    44,    44,    -4,
    41,    44,   -10,    44,   -10,   -10,   -10,    44,    41,    44,
    -1,    -5,    44,    -3,   274,    44,    -5,    44,    44,    44,
    -1,    -5,    -7,    -1,    -5,    41,    44,    -5,    41,    -5,
    44,    -1,    -1,    -5,    44,    44,    -5,    44,    44,    -1,
    44,    44,    44,    -5,    -1,    -5,    44,    -5,    -1,    41,
    -1,    -5,    -5,    41,    41,    41,    -1,    44,    41,   264,
    41,    41,    41,   264,   264,   264,    41,    -1,   264,   264,
   264,   264,    41 };
yytabelem yydef[]={

     0,    -2,     1,     0,     0,     0,    12,    62,    10,    34,
     0,    61,     0,    31,    33,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,    79,     8,     9,     2,     0,
     3,     0,     4,     0,     0,    71,    72,    73,    74,    75,
    76,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,    77,    78,     0,     0,    10,    12,    80,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,    52,    34,
     0,    62,    53,    54,     5,     6,    51,     0,    56,     0,
     0,    13,    59,    43,    44,    45,    46,    47,    48,    49,
    50,    55,    63,    64,     0,     0,     0,     0,    11,    30,
     0,     0,    57,    66,    26,    81,    27,    82,     7,     0,
     0,     0,     0,     0,     0,     0,     0,     0,    23,     0,
     0,    17,     0,    -2,     0,    -2,     0,     0,    28,    32,
    29,    60,     0,    68,    35,    36,    37,     0,     0,     0,
    41,     0,     0,     0,     0,     0,     0,     0,    65,     0,
     0,     0,     0,     0,    58,     0,     0,     0,     0,     0,
     0,     0,    67,    69,    70,    38,     0,     0,    42,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,    24,
     0,     0,     0,    15,    18,    21,    39,     0,    25,    19,
    16,    22,    40 };
typedef struct
#ifdef __cplusplus
	yytoktype
#endif
{ char *t_name; int t_val; } yytoktype;
#ifndef YYDEBUG
#	define YYDEBUG	1	/* allow debugging */
#endif

#if YYDEBUG

yytoktype yytoks[] =
{
	"IF",	257,
	"IN",	258,
	"RESERVED",	259,
	"FORMAT",	260,
	"INTERPOLATE",	261,
	"EOS",	262,
	"EOI",	263,
	"CONNECTOR",	264,
	"RANDOM",	265,
	"FLOAT",	266,
	"C_FNCT",	267,
	"S_FNCT",	268,
	"D_FNCT",	269,
	"HSG_FNCT",	270,
	"HSG_CODE_SYM",	271,
	"HSG_NUM_SYM",	272,
	"UNKNOWN",	273,
	"MULTI_STRING",	274,
	"STRING",	275,
	"INT",	276,
	"=",	61,
	"OR",	277,
	"AND",	278,
	"?",	63,
	"numc",	279,
	"EQ",	280,
	"NE",	281,
	"LT",	282,
	"LE",	283,
	"GT",	284,
	"GE",	285,
	"MINUS",	286,
	"PLUS",	287,
	"MUL",	288,
	"DIV",	289,
	"MOD",	290,
	"SIG",	291,
	"POW",	292,
	"NOT",	293,
	"UNARY",	294,
	"-unknown-",	-1	/* ends search */
};

char * yyreds[] =
{
	"-no such reduction-",
	"input : EOI",
	"input : allStrings EOI",
	"input : allExpr EOI",
	"input : error EOI",
	"input : allStrings error EOI",
	"input : allExpr error EOI",
	"empty : /* empty */",
	"constValue : FLOAT",
	"constValue : INT",
	"allExpr : expr",
	"allExpr : HSG_NUM_SYM '=' allStrings",
	"allStrings : strings",
	"allStrings : HSG_CODE_SYM '=' expr",
	"numChrCat : oneNumChrCat strings",
	"oneNumChrCat : '(' logicalRelation ',' expr ',' strings ')' CONNECTOR",
	"oneNumChrCat : oneNumChrCat '(' logicalRelation ',' expr ',' strings ')' CONNECTOR",
	"numNumCat : oneNumNumCat expr",
	"oneNumNumCat : '(' logicalRelation ',' expr ',' expr ')' CONNECTOR",
	"oneNumNumCat : oneNumNumCat '(' logicalRelation ',' expr ',' expr ')' CONNECTOR",
	"chrChrCat : oneChrChrCat strings",
	"oneChrChrCat : '(' logicalRelation ',' strings ',' strings ')' CONNECTOR",
	"oneChrChrCat : oneChrChrCat '(' logicalRelation ',' strings ',' strings ')' CONNECTOR",
	"chrNumCat : oneChrNumCat expr",
	"oneChrNumCat : '(' logicalRelation ',' strings ',' expr ')' CONNECTOR",
	"oneChrNumCat : oneChrNumCat '(' logicalRelation ',' strings ',' expr ')' CONNECTOR",
	"multiStatement : allExpr EOS",
	"multiStatement : allStrings EOS",
	"multiStatement : multiStatement allExpr EOS",
	"multiStatement : multiStatement allStrings EOS",
	"expr : HSG_NUM_SYM '=' expr",
	"expr : UNKNOWN",
	"expr : '(' multiStatement allExpr ')'",
	"expr : constValue",
	"expr : HSG_NUM_SYM",
	"expr : HSG_FNCT '(' strings ')'",
	"expr : S_FNCT '(' expr ')'",
	"expr : C_FNCT '(' strings ')'",
	"expr : D_FNCT '(' expr ',' expr ')'",
	"expr : INTERPOLATE '(' STRING ',' strings ',' strings ',' expr ')'",
	"expr : INTERPOLATE '(' STRING allStrings ',' strings ',' strings ',' expr ')'",
	"expr : RANDOM '(' STRING ')'",
	"expr : IN '(' strings ',' list ')'",
	"expr : expr logicalRelation expr",
	"expr : expr booleanRelation expr",
	"expr : expr PLUS expr",
	"expr : expr MINUS expr",
	"expr : expr MUL expr",
	"expr : expr DIV expr",
	"expr : expr POW expr",
	"expr : expr MOD expr",
	"expr : strings logicalRelation strings",
	"expr : MINUS expr",
	"expr : PLUS expr",
	"expr : NOT expr",
	"expr : expr '?' numNumCat",
	"expr : strings '?' chrNumCat",
	"expr : '(' expr ')'",
	"list : MULTI_STRING",
	"strings : HSG_CODE_SYM '=' strings",
	"strings : '(' multiStatement allStrings ')'",
	"strings : STRING",
	"strings : HSG_CODE_SYM",
	"strings : expr '?' numChrCat",
	"strings : expr '?' chrChrCat",
	"strings : FORMAT '(' strings args ')'",
	"strings : '(' strings ')'",
	"args : args ',' oneArg",
	"args : empty",
	"oneArg : expr",
	"oneArg : strings",
	"logicalRelation : EQ",
	"logicalRelation : LT",
	"logicalRelation : GT",
	"logicalRelation : GE",
	"logicalRelation : LE",
	"logicalRelation : NE",
	"booleanRelation : AND",
	"booleanRelation : OR",
	"expr : ')'",
	"expr : '(' EOI",
	"expr : '(' allExpr EOI",
	"expr : '(' allStrings EOI",
};
#endif /* YYDEBUG */
#if !defined(lint) && !defined(__cplusplus)
static  char __yaccpar_sccsid1[] = "@(#) 9/3/92 yaccpar 6.11 Copyr 1991 Sun Micro";
#endif

/*
** Skeleton parser driver for yacc output
*/

/*
** yacc user known macros and defines
*/
#define YYERROR		goto yyerrlab
#define YYACCEPT	return(0)
#define YYABORT		return(1)
#define YYBACKUP( newtoken, newvalue )\
{\
	if ( yychar >= 0 || ( yyr2[ yytmp ] >> 1 ) != 1 )\
	{\
		yyerror( "syntax error - cannot backup" );\
		goto yyerrlab;\
	}\
	yychar = newtoken;\
	yystate = *yyps;\
	yylval = newvalue;\
	goto yynewstate;\
}
#define YYRECOVERING()	(!!yyerrflag)
#define YYNEW(type)	malloc(sizeof(type) * yynewmax)
#define YYCOPY(to, from, type) \
	(type *) memcpy(to, (char *) from, yynewmax * sizeof(type))
#define YYENLARGE( from, type) \
	(type *) realloc((char *) from, yynewmax * sizeof(type))
#ifndef YYDEBUG
#	define YYDEBUG	1	/* make debugging available */
#endif

/*
** user known globals
*/
int yydebug;			/* set to 1 to get debugging */

/*
** driver internal defines
*/
#define YYFLAG		(-10000000)

/*
** global variables used by the parser
*/
YYSTYPE *yypv;			/* top of value stack */
int *yyps;			/* top of state stack */

int yystate;			/* current state */
int yytmp;			/* extra var (lasts between blocks) */

int yynerrs;			/* number of errors */
int yyerrflag;			/* error recovery flag */
int yychar;			/* current input token number */



#ifdef YYNMBCHARS
#define YYLEX()		yycvtok(yylex())
/*
** yycvtok - return a token if i is a wchar_t value that exceeds 255.
**	If i<255, i itself is the token.  If i>255 but the neither 
**	of the 30th or 31st bit is on, i is already a token.
*/
#if defined(__STDC__) || defined(__cplusplus)
int yycvtok(int i)
#else
int yycvtok(i) int i;
#endif
{
	int first = 0;
	int last = YYNMBCHARS - 1;
	int mid;
	wchar_t j;

	if(i&0x60000000){/*Must convert to a token. */
		if( yymbchars[last].character < i ){
			return i;/*Giving up*/
		}
		while ((last>=first)&&(first>=0)) {/*Binary search loop*/
			mid = (first+last)/2;
			j = yymbchars[mid].character;
			if( j==i ){/*Found*/ 
				return yymbchars[mid].tvalue;
			}else if( j<i ){
				first = mid + 1;
			}else{
				last = mid -1;
			}
		}
		/*No entry in the table.*/
		return i;/* Giving up.*/
	}else{/* i is already a token. */
		return i;
	}
}
#else/*!YYNMBCHARS*/
#define YYLEX()		yylex()
#endif/*!YYNMBCHARS*/

/*
** yyparse - return 0 if worked, 1 if syntax error not recovered from
*/
#if defined(__STDC__) || defined(__cplusplus)
int yyparse(void)
#else
int yyparse()
#endif
{
	register YYSTYPE *yypvt;	/* top of value stack for $vars */

#if defined(__cplusplus) || defined(lint)
/*
	hacks to please C++ and lint - goto's inside switch should never be
	executed; yypvt is set to 0 to avoid "used before set" warning.
*/
	static int __yaccpar_lint_hack__ = 0;
	switch (__yaccpar_lint_hack__)
	{
		case 1: goto yyerrlab;
		case 2: goto yynewstate;
	}
	yypvt = 0;
#endif

	/*
	** Initialize externals - yyparse may be called more than once
	*/
	yypv = &yyv[-1];
	yyps = &yys[-1];
	yystate = 0;
	yytmp = 0;
	yynerrs = 0;
	yyerrflag = 0;
	yychar = -1;

#if YYMAXDEPTH <= 0
	if (yymaxdepth <= 0)
	{
		if ((yymaxdepth = YYEXPAND(0)) <= 0)
		{
			yyerror("yacc initialization error");
			YYABORT;
		}
	}
#endif

	{
		register YYSTYPE *yy_pv;	/* top of value stack */
		register int *yy_ps;		/* top of state stack */
		register int yy_state;		/* current state */
		register int  yy_n;		/* internal state number info */
	goto yystack;	/* moved from 6 lines above to here to please C++ */

		/*
		** get globals into registers.
		** branch to here only if YYBACKUP was called.
		*/
	yynewstate:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;
		goto yy_newstate;

		/*
		** get globals into registers.
		** either we just started, or we just finished a reduction
		*/
	yystack:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;

		/*
		** top of for (;;) loop while no reductions done
		*/
	yy_stack:
		/*
		** put a state and value onto the stacks
		*/
#if YYDEBUG
		/*
		** if debugging, look up token value in list of value vs.
		** name pairs.  0 and negative (-1) are special values.
		** Note: linear search is used since time is not a real
		** consideration while debugging.
		*/
		if ( yydebug )
		{
			register int yy_i;

			printf( "State %d, token ", yy_state );
			if ( yychar == 0 )
				printf( "end-of-file\n" );
			else if ( yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ++yy_ps >= &yys[ yymaxdepth ] )	/* room on stack? */
		{
			/*
			** reallocate and recover.  Note that pointers
			** have to be reset, or bad things will happen
			*/
			int yyps_index = (yy_ps - yys);
			int yypv_index = (yy_pv - yyv);
			int yypvt_index = (yypvt - yyv);
			int yynewmax;
#ifdef YYEXPAND
			yynewmax = YYEXPAND(yymaxdepth);
#else
			yynewmax = 2 * yymaxdepth;	/* double table size */
			if (yymaxdepth == YYMAXDEPTH)	/* first time growth */
			{
				char *newyys = (char *)YYNEW(int);
				char *newyyv = (char *)YYNEW(YYSTYPE);
				if (newyys != 0 && newyyv != 0)
				{
					yys = YYCOPY(newyys, yys, int);
					yyv = YYCOPY(newyyv, yyv, YYSTYPE);
				}
				else
					yynewmax = 0;	/* failed */
			}
			else				/* not first time */
			{
				yys = YYENLARGE(yys, int);
				yyv = YYENLARGE(yyv, YYSTYPE);
				if (yys == 0 || yyv == 0)
					yynewmax = 0;	/* failed */
			}
#endif
			if (yynewmax <= yymaxdepth)	/* tables not expanded */
			{
				yyerror( "yacc stack overflow" );
				YYABORT;
			}
			yymaxdepth = yynewmax;

			yy_ps = yys + yyps_index;
			yy_pv = yyv + yypv_index;
			yypvt = yyv + yypvt_index;
		}
		*yy_ps = yy_state;
		*++yy_pv = yyval;

		/*
		** we have a new state - find out what to do
		*/
	yy_newstate:
		if ( ( yy_n = yypact[ yy_state ] ) <= YYFLAG )
			goto yydefault;		/* simple state */
#if YYDEBUG
		/*
		** if debugging, need to mark whether new token grabbed
		*/
		yytmp = yychar < 0;
#endif
		if ( ( yychar < 0 ) && ( ( yychar = YYLEX() ) < 0 ) )
			yychar = 0;		/* reached EOF */
#if YYDEBUG
		if ( yydebug && yytmp )
		{
			register int yy_i;

			printf( "Received token " );
			if ( yychar == 0 )
				printf( "end-of-file\n" );
			else if ( yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ( ( yy_n += yychar ) < 0 ) || ( yy_n >= YYLAST ) )
			goto yydefault;
		if ( yychk[ yy_n = yyact[ yy_n ] ] == yychar )	/*valid shift*/
		{
			yychar = -1;
			yyval = yylval;
			yy_state = yy_n;
			if ( yyerrflag > 0 )
				yyerrflag--;
			goto yy_stack;
		}

	yydefault:
		if ( ( yy_n = yydef[ yy_state ] ) == -2 )
		{
#if YYDEBUG
			yytmp = yychar < 0;
#endif
			if ( ( yychar < 0 ) && ( ( yychar = YYLEX() ) < 0 ) )
				yychar = 0;		/* reached EOF */
#if YYDEBUG
			if ( yydebug && yytmp )
			{
				register int yy_i;

				printf( "Received token " );
				if ( yychar == 0 )
					printf( "end-of-file\n" );
				else if ( yychar < 0 )
					printf( "-none-\n" );
				else
				{
					for ( yy_i = 0;
						yytoks[yy_i].t_val >= 0;
						yy_i++ )
					{
						if ( yytoks[yy_i].t_val
							== yychar )
						{
							break;
						}
					}
					printf( "%s\n", yytoks[yy_i].t_name );
				}
			}
#endif /* YYDEBUG */
			/*
			** look through exception table
			*/
			{
				register int *yyxi = yyexca;

				while ( ( *yyxi != -1 ) ||
					( yyxi[1] != yy_state ) )
				{
					yyxi += 2;
				}
				while ( ( *(yyxi += 2) >= 0 ) &&
					( *yyxi != yychar ) )
					;
				if ( ( yy_n = yyxi[1] ) < 0 )
					YYACCEPT;
			}
		}

		/*
		** check for syntax error
		*/
		if ( yy_n == 0 )	/* have an error */
		{
			/* no worry about speed here! */
			switch ( yyerrflag )
			{
			case 0:		/* new error */
				yyerror( "syntax error" );
				goto skip_init;
			yyerrlab:
				/*
				** get globals into registers.
				** we have a user generated syntax type error
				*/
				yy_pv = yypv;
				yy_ps = yyps;
				yy_state = yystate;
			skip_init:
				yynerrs++;
				/* FALLTHRU */
			case 1:
			case 2:		/* incompletely recovered error */
					/* try again... */
				yyerrflag = 3;
				/*
				** find state where "error" is a legal
				** shift action
				*/
				while ( yy_ps >= yys )
				{
					yy_n = yypact[ *yy_ps ] + YYERRCODE;
					if ( yy_n >= 0 && yy_n < YYLAST &&
						yychk[yyact[yy_n]] == YYERRCODE)					{
						/*
						** simulate shift of "error"
						*/
						yy_state = yyact[ yy_n ];
						goto yy_stack;
					}
					/*
					** current state has no shift on
					** "error", pop stack
					*/
#if YYDEBUG
#	define _POP_ "Error recovery pops state %d, uncovers state %d\n"
					if ( yydebug )
						printf( _POP_, *yy_ps,
							yy_ps[-1] );
#	undef _POP_
#endif
					yy_ps--;
					yy_pv--;
				}
				/*
				** there is no state on stack with "error" as
				** a valid shift.  give up.
				*/
				YYABORT;
			case 3:		/* no shift yet; eat a token */
#if YYDEBUG
				/*
				** if debugging, look up token in list of
				** pairs.  0 and negative shouldn't occur,
				** but since timing doesn't matter when
				** debugging, it doesn't hurt to leave the
				** tests here.
				*/
				if ( yydebug )
				{
					register int yy_i;

					printf( "Error recovery discards " );
					if ( yychar == 0 )
						printf( "token end-of-file\n" );
					else if ( yychar < 0 )
						printf( "token -none-\n" );
					else
					{
						for ( yy_i = 0;
							yytoks[yy_i].t_val >= 0;
							yy_i++ )
						{
							if ( yytoks[yy_i].t_val
								== yychar )
							{
								break;
							}
						}
						printf( "token %s\n",
							yytoks[yy_i].t_name );
					}
				}
#endif /* YYDEBUG */
				if ( yychar == 0 )	/* reached EOF. quit */
					YYABORT;
				yychar = -1;
				goto yy_newstate;
			}
		}/* end if ( yy_n == 0 ) */
		/*
		** reduction by production yy_n
		** put stack tops, etc. so things right after switch
		*/
#if YYDEBUG
		/*
		** if debugging, print the string that is the user's
		** specification of the reduction which is just about
		** to be done.
		*/
		if ( yydebug )
			printf( "Reduce by (%d) \"%s\"\n",
				yy_n, yyreds[ yy_n ] );
#endif
		yytmp = yy_n;			/* value to switch over */
		yypvt = yy_pv;			/* $vars top of value stack */
		/*
		** Look in goto table for next state
		** Sorry about using yy_state here as temporary
		** register variable, but why not, if it works...
		** If yyr2[ yy_n ] doesn't have the low order bit
		** set, then there is no action to be done for
		** this reduction.  So, no saving & unsaving of
		** registers done.  The only difference between the
		** code just after the if and the body of the if is
		** the goto yy_stack in the body.  This way the test
		** can be made before the choice of what to do is needed.
		*/
		{
			/* length of production doubled with extra bit */
			register int yy_len = yyr2[ yy_n ];

			if ( !( yy_len & 01 ) )
			{
				yy_len >>= 1;
				yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
				yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
					*( yy_ps -= yy_len ) + 1;
				if ( yy_state >= YYLAST ||
					yychk[ yy_state =
					yyact[ yy_state ] ] != -yy_n )
				{
					yy_state = yyact[ yypgo[ yy_n ] ];
				}
				goto yy_stack;
			}
			yy_len >>= 1;
			yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
			yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
				*( yy_ps -= yy_len ) + 1;
			if ( yy_state >= YYLAST ||
				yychk[ yy_state = yyact[ yy_state ] ] != -yy_n )
			{
				yy_state = yyact[ yypgo[ yy_n ] ];
			}
		}
					/* save until reenter driver code */
		yystate = yy_state;
		yyps = yy_ps;
		yypv = yy_pv;
	}
	/*
	** code supplied by user is placed in this switch
	*/
	switch( yytmp )
	{
		
case 1:
# line 302 "parser.y"
{  return 0; } break;
case 2:
# line 305 "parser.y"
{
expr = yypvt[-1].node;
YYACCEPT;
} break;
case 3:
# line 311 "parser.y"
{
expr = yypvt[-1].node;
YYACCEPT;
} break;
case 4:
# line 317 "parser.y"
{
  YYABORT;
} break;
case 5:
# line 321 "parser.y"
{
  YYABORT;
} break;
case 6:
# line 324 "parser.y"
{
  YYABORT;
} break;
case 7:
# line 332 "parser.y"
{;} break;
case 8:
# line 338 "parser.y"
{ yyval.value = yypvt[-0].value;} break;
case 9:
# line 339 "parser.y"
{ yyval.value = (double) yypvt[-0].integer;} break;
case 10:
# line 347 "parser.y"
{
  yyval.node = yypvt[-0].node;
} break;
case 11:
# line 352 "parser.y"
{
Sdberror("cannot assign code value to numeric variable");
YYERROR;
} break;
case 12:
# line 363 "parser.y"
{
  yyval.node = yypvt[-0].node;
} break;
case 13:
# line 368 "parser.y"
{
Sdberror("cannot assign numeric value to code variable");
YYERROR;
} break;
case 14:
# line 380 "parser.y"
{
   SdbCat *pntr, *newclass=(SdbCat*) ckalloc(sizeof(SdbCat));

   /* create a new character node */
   if (!newclass) {
     fprintf(stderr,"Out of memory\n");
     exit(1);
   }

   /* add the default value to the end of the list of categories */
   for (pntr = yypvt[-1].classList; pntr->next; pntr=pntr->next);

   /* set the appropriate values */
   pntr->next = newclass;
   newclass->expr = NULL;
   newclass->relation = opNop;
   newclass->code = yypvt[-0].node;
   newclass->next = NULL;

   /* return the pointer to the beginning of the structure */
   yyval.classList = yypvt[-1].classList;
 } break;
case 15:
# line 408 "parser.y"
{
   SdbCat *newclass= safeGetCat();

   if (!newclass) {
     fprintf(stderr,"Out of memory\n");
     exit(1);
   }
   
   newclass->code = yypvt[-2].node;
   newclass->relation = yypvt[-6].op;
   newclass->expr = yypvt[-4].node;
   newclass->next = NULL;
   yyval.classList = newclass;
   
 } break;
case 16:
# line 424 "parser.y"
{    
   SdbCat *pntr, *newclass=(SdbCat*) ckalloc(sizeof(SdbCat));
 
   if (!newclass) {
     fprintf(stderr,"Out of memory\n");
     exit(1);
   }
   /* add this to the end of the previous list */
   for (pntr = yypvt[-8].classList; pntr->next; pntr=pntr->next);

   pntr->next=newclass;
   newclass->code = yypvt[-2].node;
   newclass->relation = yypvt[-6].op;
   newclass->expr = yypvt[-4].node;
   newclass->next = NULL;
   yyval.classList = yypvt[-8].classList;
   } break;
case 17:
# line 449 "parser.y"
{
   SdbCat *pntr, *newclass=(SdbCat*)ckalloc(sizeof(SdbCat));

   /* create a new character node */
   if (!newclass) {
     fprintf(stderr,"Out of memory\n");
     exit(1);
   }

   /* add the default value to the end of the list of categories */
   for (pntr = yypvt[-1].classList; pntr->next; pntr=pntr->next);

   /* set the appropriate values */
   pntr->next = newclass;
   newclass->expr = NULL;
   newclass->relation = opNop;
   newclass->code = yypvt[-0].node;
   newclass->next = NULL;

   /* return the pointer to the beginning of the structure */
   yyval.classList = yypvt[-1].classList;
 } break;
case 18:
# line 473 "parser.y"
{
   SdbCat *newclass = safeGetCat();

   if (!newclass) {
     fprintf(stderr,"Out of memory\n");
     exit(1);
   }
   
   newclass->code = yypvt[-2].node;
   newclass->relation = yypvt[-6].op;
   newclass->expr = yypvt[-4].node;
   newclass->next = NULL;
   yyval.classList = newclass;
   
 } break;
case 19:
# line 489 "parser.y"
{    
   SdbCat *pntr,*newclass= (SdbCat*)ckalloc(sizeof(SdbCat));

   if (!newclass) {
     fprintf(stderr,"Out of memory\n");
     exit(1);
   }
   /* add this to the end of the previous list */
   for (pntr = yypvt[-8].classList; pntr->next; pntr=pntr->next);

   pntr->next=newclass;
   newclass->code = yypvt[-2].node;
   newclass->relation = yypvt[-6].op;
   newclass->expr = yypvt[-4].node;
   newclass->next = NULL;
   yyval.classList = yypvt[-8].classList;
   } break;
case 20:
# line 513 "parser.y"
{
   SdbCat *pntr, *newclass=(SdbCat*) ckalloc(sizeof(SdbCat));

   /* create a new character node */
   if (!newclass) {
     fprintf(stderr,"Out of memory\n");
     exit(1);
   }

   /* add the default value to the end of the list of categories */
   for (pntr = yypvt[-1].classList; pntr->next; pntr=pntr->next);

   /* set the appropriate values */
   pntr->next = newclass;
   newclass->expr = NULL;
   newclass->relation = opNop;
   newclass->code = yypvt[-0].node;
   newclass->next = NULL;

   /* return the pointer to the beginning of the structure */
   yyval.classList = yypvt[-1].classList;
 } break;
case 21:
# line 541 "parser.y"
{
   SdbCat *newclass= safeGetCat();

   if (!newclass) {
     fprintf(stderr,"Out of memory\n");
     exit(1);
   }
   
   newclass->code = yypvt[-2].node;
   newclass->relation = yypvt[-6].op;
   newclass->expr = yypvt[-4].node;
   newclass->next = NULL;
   yyval.classList = newclass;
   
 } break;
case 22:
# line 557 "parser.y"
{    
   SdbCat *pntr, *newclass=(SdbCat*) ckalloc(sizeof(SdbCat));
 
   if (!newclass) {
     fprintf(stderr,"Out of memory\n");
     exit(1);
   }
   /* add this to the end of the previous list */
   for (pntr = yypvt[-8].classList; pntr->next; pntr=pntr->next);

   pntr->next=newclass;
   newclass->code = yypvt[-2].node;
   newclass->relation = yypvt[-6].op;
   newclass->expr = yypvt[-4].node;
   newclass->next = NULL;
   yyval.classList = yypvt[-8].classList;
   } break;
case 23:
# line 582 "parser.y"
{
   SdbCat *pntr, *newclass=(SdbCat*)ckalloc(sizeof(SdbCat));

   /* create a new character node */
   if (!newclass) {
     fprintf(stderr,"Out of memory\n");
     exit(1);
   }

   /* add the default value to the end of the list of categories */
   for (pntr = yypvt[-1].classList; pntr->next; pntr=pntr->next);

   /* set the appropriate values */
   pntr->next = newclass;
   newclass->expr = NULL;
   newclass->relation = opNop;
   newclass->code = yypvt[-0].node;
   newclass->next = NULL;

   /* return the pointer to the beginning of the structure */
   yyval.classList = yypvt[-1].classList;
 } break;
case 24:
# line 606 "parser.y"
{
   SdbCat *newclass = safeGetCat();

   if (!newclass) {
     fprintf(stderr,"Out of memory\n");
     exit(1);
   }
   
   newclass->code = yypvt[-2].node;
   newclass->relation = yypvt[-6].op;
   newclass->expr = yypvt[-4].node;
   newclass->next = NULL;
   yyval.classList = newclass;
   
 } break;
case 25:
# line 622 "parser.y"
{    
   SdbCat *pntr,*newclass= (SdbCat*)ckalloc(sizeof(SdbCat));

   if (!newclass) {
     fprintf(stderr,"Out of memory\n");
     exit(1);
   }
   /* add this to the end of the previous list */
   for (pntr = yypvt[-8].classList; pntr->next; pntr=pntr->next);

   pntr->next=newclass;
   newclass->code = yypvt[-2].node;
   newclass->relation = yypvt[-6].op;
   newclass->expr = yypvt[-4].node;
   newclass->next = NULL;
   yyval.classList = yypvt[-8].classList;
   } break;
case 26:
# line 651 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = numSequence;
  newnode->left = NULL;
  newnode->right = yypvt[-1].node;
  yyval.node = newnode;
} break;
case 27:
# line 660 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = charSequence;
  newnode->left = NULL;
  newnode->right = yypvt[-1].node;
  yyval.node = newnode;
} break;
case 28:
# line 669 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = numSequence;
  newnode->left = yypvt[-2].node;
  newnode->right = yypvt[-1].node;
  yyval.node = newnode;
} break;
case 29:
# line 678 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = charSequence;
  newnode->left = yypvt[-2].node;
  newnode->right = yypvt[-1].node;
  yyval.node = newnode;
} break;
case 30:
# line 694 "parser.y"
{
  SdbNode *subnode, *newnode=safeGetNode();
  newnode->type = assignNum;
  newnode->right = yypvt[-0].node;
  subnode = safeGetNode();
  subnode->type = hsgNumber;
  subnode->content.code = yypvt[-2].string;
  subnode->right=NULL;
  subnode->left=NULL;
  newnode->left=subnode;
  yyval.node = newnode;
} break;
case 31:
# line 708 "parser.y"
{
  Sdberror("cannot use unknown symbol in expression");
  YYERROR;
} break;
case 32:
# line 714 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = numSequence;
  newnode->left = yypvt[-2].node;
  newnode->right = yypvt[-1].node;
  yyval.node = newnode;
} break;
case 33:
# line 723 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = constValue;
  newnode->content.value = yypvt[-0].value;
  yyval.node = newnode;
} break;
case 34:
# line 731 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = hsgNumber;
  newnode->content.code = yypvt[-0].string;
  yyval.node = newnode;
} break;
case 35:
# line 739 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = hsgFnct;
  newnode->content.fnctptr = yypvt[-3].fnctptr;
  newnode->right = yypvt[-1].node;
  yyval.node = newnode;
} break;
case 36:
# line 748 "parser.y"
{ 
  SdbNode *newnode=safeGetNode();
  newnode->type = simplexFnct;
  newnode->content.fnctptr = yypvt[-3].fnctptr;
  newnode->right = yypvt[-1].node;
  yyval.node = newnode;
} break;
case 37:
# line 757 "parser.y"
{ 
  SdbNode *newnode=safeGetNode();
  newnode->type = charFnct;
  newnode->content.fnctptr = yypvt[-3].fnctptr;
  newnode->right = yypvt[-1].node;
  yyval.node = newnode;
} break;
case 38:
# line 766 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = duplexFnct;
  newnode->content.fnctptr = yypvt[-5].fnctptr;
  newnode->left = yypvt[-3].node;
  newnode->right= yypvt[-1].node;
  yyval.node = newnode;
} break;
case 39:
# line 776 "parser.y"
{
  SdbNode *newnode=safeGetNode(), *subnode, *tempnode;
  newnode->type = curveFnct;
  newnode->content.yieldHandle = 
    (SdbYieldHandle*)ckalloc(sizeof(SdbYieldHandle));
  (newnode->content.yieldHandle)->xIndex = -1;
  (newnode->content.yieldHandle)->yIndex = -1;
  (newnode->content.yieldHandle)->yieldSet = NULL;
  (newnode->content.yieldHandle)->list     = NULL;
  (newnode->content.yieldHandle)->stamp    = 0;
  subnode=safeGetNode();
  tempnode= safeGetNode();
  tempnode->type = noScan;
  tempnode->left=safeGetNode();
  tempnode->left->type = constCode;
  tempnode->left->content.code = yypvt[-7].string;
  tempnode->right = NULL;
  subnode->type = binaryOp;
  subnode->content.op =opNop;
  subnode->left=tempnode;
  subnode->right=yypvt[-5].node;
  newnode->left = subnode;
  subnode=safeGetNode();
  subnode->type = binaryOp;
  subnode->content.op =opNop;
  subnode->left=yypvt[-3].node;
  subnode->right=yypvt[-1].node;
  newnode->right = subnode;
  yyval.node = newnode;
} break;
case 40:
# line 807 "parser.y"
{
  SdbNode *newnode, *subnode, *tempnode;
  
  /*
   * Before accepting this node, make sure that the $4 expr is a
   * character sequence consisting only of simple variable assignments.
   */
  tempnode=yypvt[-7].node;
  if (tempnode->type == assignChar) {
    if (tempnode->right->type == hsgCode) 
      /* ok, do nothing */;
    else {
      Sdberror("lookup alias must be simple variable assignment");
      YYERROR;
    }
  }
  else if (tempnode->type == charSequence) {
    SdbNode *next=tempnode;
    while (next) {
      if (!(next->right->type == assignChar &&
	    next->right->right->type == hsgCode)) {
	Sdberror("lookup alias must be sequence of simple variable assignments");
	YYERROR;
      }
      next = next->left;
    }
  }
  else {
    Sdberror("lookup alias must be sequence of simple variable assignments");
    YYERROR;
  }

  /*
   * The alias expression looks ok.  Process the rest of the 
   * record.
   */

  newnode=safeGetNode();
  newnode->type = curveFnct;
  newnode->content.yieldHandle = 
    (SdbYieldHandle*)ckalloc(sizeof(SdbYieldHandle));
  (newnode->content.yieldHandle)->xIndex = -1;
  (newnode->content.yieldHandle)->yIndex = -1;
  (newnode->content.yieldHandle)->yieldSet = NULL;
  (newnode->content.yieldHandle)->list     = NULL;
  (newnode->content.yieldHandle)->stamp    = 0;
  subnode=safeGetNode();
  tempnode= safeGetNode();
  tempnode->type = noScan;
  tempnode->left=safeGetNode();
  tempnode->left->type = constCode;
  tempnode->left->content.code = yypvt[-8].string;
  tempnode->right = yypvt[-7].node;
  subnode->type = binaryOp;
  subnode->content.op =opNop;
  subnode->left=tempnode;
  subnode->right=yypvt[-5].node;
  newnode->left = subnode;
  subnode=safeGetNode();
  subnode->type = binaryOp;
  subnode->content.op =opNop;
  subnode->left=yypvt[-3].node;
  subnode->right=yypvt[-1].node;
  newnode->right = subnode;
  yyval.node = newnode;
} break;
case 41:
# line 875 "parser.y"
{
  SdbNode *subnode=safeGetNode(), *newnode=safeGetNode();
  newnode->type = randomFnct;
  newnode->content.randomPtr = NULL;
  subnode->type = constCode;
  subnode->content.code = yypvt[-1].string;
  newnode->right = subnode;
  newnode->left=NULL;
  yyval.node = newnode;
} break;
case 42:
# line 887 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = stringFind;
  newnode->left = yypvt[-3].node;
  newnode->right= yypvt[-1].node;
  yyval.node = newnode;
} break;
case 43:
# line 896 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = binaryOp;
  newnode->content.op = yypvt[-1].op;
  newnode->left = yypvt[-2].node;
  newnode->right= yypvt[-0].node;
  yyval.node = newnode;
} break;
case 44:
# line 906 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = booleanOp;
  newnode->content.op = yypvt[-1].op;
  newnode->left = yypvt[-2].node;
  newnode->right= yypvt[-0].node;
  yyval.node = newnode;
} break;
case 45:
# line 916 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = binaryOp;
  newnode->content.op = opSum;
  newnode->left = yypvt[-2].node;
  newnode->right= yypvt[-0].node;
  yyval.node = newnode;
} break;
case 46:
# line 926 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = binaryOp;
  newnode->content.op = opSub;
  newnode->left = yypvt[-2].node;
  newnode->right= yypvt[-0].node;
  yyval.node = newnode;
} break;
case 47:
# line 936 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = binaryOp;
  newnode->content.op = opMul;
  newnode->left = yypvt[-2].node;
  newnode->right= yypvt[-0].node;
  yyval.node = newnode;
} break;
case 48:
# line 946 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = binaryOp;
  newnode->content.op = opDiv;
  newnode->left =yypvt[-2].node;
  newnode->right=yypvt[-0].node;
  yyval.node = newnode;
} break;
case 49:
# line 956 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = duplexFnct;
  newnode->content.fnctptr = pow;
  newnode->left = yypvt[-2].node;
  newnode->right= yypvt[-0].node;
  yyval.node = newnode;
} break;
case 50:
# line 966 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = duplexFnct;
  newnode->content.fnctptr = fmod;
  newnode->left = yypvt[-2].node;
  newnode->right= yypvt[-0].node;
  yyval.node = newnode;
} break;
case 51:
# line 976 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = stringCompare;
  newnode->content.op = yypvt[-1].op;
  newnode->left = yypvt[-2].node;
  newnode->right= yypvt[-0].node;
  yyval.node = newnode;
} break;
case 52:
# line 986 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = unaryOp; 
  newnode->content.op = opNeg;
  newnode->right = yypvt[-0].node;
  yyval.node = newnode;
} break;
case 53:
# line 995 "parser.y"
{
 yyval.node = yypvt[-0].node;
} break;
case 54:
# line 1000 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = unaryOp;
  newnode->content.op = opNot;
  newnode->right= yypvt[-0].node;
  yyval.node = newnode;
} break;
case 55:
# line 1009 "parser.y"
{  
  SdbNode *newnode=safeGetNode();
  newnode->type = numNumFnct;
  newnode->left = yypvt[-2].node;
  newnode->content.classList = yypvt[-0].classList;
  yyval.node = newnode;
} break;
case 56:
# line 1018 "parser.y"
{  
  SdbNode *newnode=safeGetNode();
  newnode->type = chrNumFnct;
  newnode->left = yypvt[-2].node;
  newnode->content.classList = yypvt[-0].classList;
  yyval.node = newnode;
} break;
case 57:
# line 1027 "parser.y"
{ 
  yyval.node = yypvt[-1].node; 
} break;
case 58:
# line 1038 "parser.y"
{
   SdbNode *newnode = safeGetNode();
   char **pntr, *code, *token = yypvt[-0].string;
   int k=1, new;

   /* make space for a fixed length code list */
   for (code=token; code[0] != 0; code++) 
     if (code[0] == '/') k++;

   if (!(pntr = (char**)ckalloc(k*sizeof(char*)))) {
     fprintf(stderr,"Out of memory\n");
     exit(1);
   }
   
   /* loop through and load up the list */
   k = 0;
   while (token && ((code = strtok (token, "/")) != NULL)) {
     pntr[k] = GetIndices (htbl,code, &new);
     token = strtok (NULL, "");
     k++;
   }
   newnode->type = hsgCodeList;
   newnode->content.list.n = k;
   newnode->content.list.pntr = pntr;
   ckfree(yypvt[-0].string);
   yyval.node = newnode;
 } break;
case 59:
# line 1075 "parser.y"
{
  SdbNode *subnode, *newnode=safeGetNode();
  newnode->type = assignChar;
  newnode->right = yypvt[-0].node;
  subnode = safeGetNode();
  subnode->type = hsgCode;
  subnode->content.code = yypvt[-2].string;
  subnode->right=NULL;
  subnode->left=NULL;
  newnode->left=subnode;
  yyval.node = newnode;
} break;
case 60:
# line 1089 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = charSequence;
  newnode->left = yypvt[-2].node;
  newnode->right = yypvt[-1].node;
  yyval.node = newnode;
} break;
case 61:
# line 1098 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = constCode;
  newnode->content.code = yypvt[-0].string;
  yyval.node = newnode;
} break;
case 62:
# line 1106 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = hsgCode;
  newnode->content.code = yypvt[-0].string;
  yyval.node = newnode;
} break;
case 63:
# line 1114 "parser.y"
{  
  SdbNode *newnode=safeGetNode();
  newnode->type = numChrFnct;
  newnode->left = yypvt[-2].node;
  newnode->content.classList = yypvt[-0].classList;
  yyval.node = newnode;
} break;
case 64:
# line 1123 "parser.y"
{  
  SdbNode *newnode=safeGetNode();
  newnode->type = chrChrFnct;
  newnode->left = yypvt[-2].node;
  newnode->content.classList = yypvt[-0].classList;
  yyval.node = newnode;
} break;
case 65:
# line 1132 "parser.y"
{
/* This function takes a fixed string as input.  It could be a variable
   string, but that would complicate things (especially if it were another
   format function, because the format function is not re-entrant) */
  SdbNode *newnode=safeGetNode();
  newnode->type = formatFnct;
  newnode->left = yypvt[-2].node;
  newnode->right = yypvt[-1].node;
  yyval.node = newnode;
} break;
case 66:
# line 1144 "parser.y"
{ 
  yyval.node = yypvt[-1].node; 
} break;
case 67:
# line 1160 "parser.y"
{

   if (yypvt[-2].node) {
     yypvt[-2].node->right = yypvt[-0].node;
     yyval.node = yypvt[-2].node;
   }
   else
     yyval.node = yypvt[-0].node;
   
 } break;
case 68:
# line 1171 "parser.y"
{ yyval.node = NULL; } break;
case 69:
# line 1174 "parser.y"
{
  SdbNode *newnode=safeGetNode();
  newnode->type = numExpr;
  newnode->left = yypvt[-0].node;
  newnode->right = NULL;
  yyval.node = newnode;

} break;
case 70:
# line 1182 "parser.y"
{  
  SdbNode *newnode=safeGetNode();
  newnode->type = charExpr;
  newnode->left = yypvt[-0].node;
  newnode->right = NULL;
  yyval.node = newnode;
} break;
case 71:
# line 1196 "parser.y"
{ yyval.op = opEq; } break;
case 72:
# line 1197 "parser.y"
{ yyval.op = opLt; } break;
case 73:
# line 1198 "parser.y"
{ yyval.op = opGt; } break;
case 74:
# line 1199 "parser.y"
{ yyval.op = opGe; } break;
case 75:
# line 1200 "parser.y"
{ yyval.op = opLe; } break;
case 76:
# line 1201 "parser.y"
{ yyval.op = opNe; } break;
case 77:
# line 1205 "parser.y"
{ yyval.op = opAnd; } break;
case 78:
# line 1206 "parser.y"
{ yyval.op = opOr; } break;
case 79:
# line 1211 "parser.y"
{
   Sdberror("unmatched right parenthesis");
   YYERROR;
 } break;
case 80:
# line 1216 "parser.y"
{
  Sdberror("unmatched left parenthesis");
  YYERROR;
} break;
case 81:
# line 1221 "parser.y"
{
  Sdberror("unmatched left parenthesis");
  YYERROR;
} break;
case 82:
# line 1226 "parser.y"
{
  Sdberror("unmatched left parenthesis");
  YYERROR;
} break;
	}
	goto yystack;		/* reset registers in driver code */
}

